<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use HCM\Util\{ DateUtil, HCMParameters, Messages, Repositories, Util };

class QuadroVagasService {
    
    const CALCULATED_FIELDS = array('QTD_VINC_ATIVOS_OCUPACAO', 'QTD_VAGAS');
    const ON_SAVE_INVALIDATION_CHECKS = [
        ['action' => 'isValidadeFimInferiorInicio', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_VALIDADE_INVALIDA],
        ['action' => 'isLimiteMenorIgualZero', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_LIMITE_INVALIDO],
        ['action' => 'isEstruturaRepetida', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_QUADRO_REPETIDO],
    ];

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function includeNewRowsData(array $rows): array {
        $alteredRows = $this->incluiCamposCalculadosRows($rows);
        return $alteredRows;
    }
    
    public function filterNewRowsData(array $rows, FilterCriteria $filterData): array {
        $filterConditions = Util::removerCaracterePorcentagemString($filterData->getConditions());
        foreach ($filterConditions as $filter) {
            $filterName = $filter['columnName'];
            $filterValue = $filter['value'];
            
            if (!in_array($filterName, self::CALCULATED_FIELDS)) {
                continue;
            }
            
            $rows = array_filter($rows, function ($item) use ($filterName, $filterValue) {
                $itemValue = $item[$filterName] ?? null;
                return $itemValue === intval($filterValue);
            });
        }
        
        return array_values($rows);
    }
    
    public function incluiCamposCalculadosRows(array $rows): array {
        $alteredRows = array_map('self::incluiCamposCalculadosRow', $rows);
        return $alteredRows;
    }
    
    public function incluiCamposCalculadosRow(array $row): array {
        $qntVinculosAtivosOcupEstrut = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQntVinculosAtivosOcupEstrutHierarq(
            $row['NRORG'],
            $this->hcmParameters->getDtCompetencia(),
            $row['NROCUPACAOM'],
            $row['NRESTRUTURAM']
        );
        
        $row['QTD_VINC_ATIVOS_OCUPACAO'] = $qntVinculosAtivosOcupEstrut;
        $row['QTD_VAGAS'] = $row['QTDLIMITE'] - $row['QTD_VINC_ATIVOS_OCUPACAO'];
        $row['QTD_VAGAS'] = $row['QTD_VAGAS'] < 0 ? 0 : $row['QTD_VAGAS'];
        
        return $row;
    }
    
    public function areRowsInvalid(array $rows, DTO\Response $response): bool {
        foreach ($rows as $row) {
            if ($this->isRowInvalid($row, $response)) {
                return true;
            }
        }
        
        return false;
    }
    
    public function isRowInvalid(object $row, DTO\Response $response): bool {
        $isRowInvalid = false;
        
        foreach (self::ON_SAVE_INVALIDATION_CHECKS as $validation) {
            $isCheckInvalid = $validation['action'];
            if ($this->$isCheckInvalid($row)) {
                $isRowInvalid = true;
                $response->addMessage(new Message($validation['invalidationMessage'], DTO\Response\Message::TYPE_WARNING));
            }
        }
        
        return $isRowInvalid;
    }
    
    public function isValidadeFimInferiorInicio(object $row): bool {
        $dataIni = DateUtil::getDataDeString(substr($row['DTINIVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $dataFim = DateUtil::getDataDeString(substr($row['DTFIMVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        return !empty($dataFim) && $dataFim < $dataIni;
    }
    
    public function isLimiteMenorIgualZero(object $row): bool {
        $dataIni = $row['QTDLIMITE'];
        return $dataIni <= 0;
    }
    
    public function isEstruturaRepetida(object $row): bool {
        $quadrosEstruturasOcupacoesIguais = $this->getQuadrosPotencialmenteRepetidos($row);
        
        $iniValidadeRow = DateUtil::getDataDeString(substr($row['DTINIVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $fimValidadeRow = DateUtil::getDataDeString(substr($row['DTFIMVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        
        foreach ($quadrosEstruturasOcupacoesIguais as $quadro) {
            $isPropostaIgual = !empty($quadro->getCdproposta()) && $quadro->getCdproposta() == $row['CDPROPOSTA'];
            $isPeriodosIntercedentes =
                (empty($fimValidadeRow) || $quadro->getDtinivalidade() <= $fimValidadeRow) && (empty($quadro->getDtfimvalidade()) || $quadro->getDtfimvalidade() >= $iniValidadeRow);
                
            if ($isPropostaIgual || $isPeriodosIntercedentes) {
                return true;
            }
        }
        
        return false;
    }
    
    public function getQuadrosPotencialmenteRepetidos(object $row): array {
        $quadrosEstruturasOcupacoesIguais = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->findBy(
            ['nrorg' => $row['NRORG'], 'nrestruturam' => $row['NRESTRUTURAM'], 'nrocupacaom' => $row['NROCUPACAOM'], 'idativo' => 'S']
        );
        
        $quadrosEstruturasOcupacoesIguais = array_filter(
            $quadrosEstruturasOcupacoesIguais,
            function ($item) use ($row) {
                return $item->getNrquadrovagas() != ($row['NRQUADROVAGAS'] ?? null);
            }
        );
        
        return $quadrosEstruturasOcupacoesIguais;
    }
}